@@ -1,17 +1,52 @@
 //! Builder-pattern objects for configuration various git operations.
 
+use libc::{c_char, c_int, c_uint, c_void, size_t};
 use std::ffi::{CStr, CString};
 use std::mem;
 use std::path::Path;
 use std::ptr;
-use libc::{c_char, size_t, c_void, c_uint, c_int};
 
-use {raw, panic, Error, Repository, FetchOptions, IntoCString};
-use {CheckoutNotificationType, DiffFile, Remote};
-use util::{self, Binding};
+use crate::util::{self, Binding};
+use crate::{panic, raw, Error, FetchOptions, IntoCString, Oid, Repository, Tree};
+use crate::{CheckoutNotificationType, DiffFile, FileMode, Remote};
 
 /// A builder struct which is used to build configuration for cloning a new git
 /// repository.
+///
+/// # Example
+///
+/// Cloning using SSH:
+///
+/// ```no_run
+/// use git2::{Cred, Error, RemoteCallbacks};
+/// use std::env;
+/// use std::path::Path;
+///
+///   // Prepare callbacks.
+///   let mut callbacks = RemoteCallbacks::new();
+///   callbacks.credentials(|_url, username_from_url, _allowed_types| {
+///     Cred::ssh_key(
+///       username_from_url.unwrap(),
+///       None,
+///       Path::new(&format!("{}/.ssh/id_rsa", env::var("HOME").unwrap())),
+///       None,
+///     )
+///   });
+///
+///   // Prepare fetch options.
+///   let mut fo = git2::FetchOptions::new();
+///   fo.remote_callbacks(callbacks);
+///
+///   // Prepare builder.
+///   let mut builder = git2::build::RepoBuilder::new();
+///   builder.fetch_options(fo);
+///
+///   // Clone the project.
+///   builder.clone(
+///     "git@github.com:rust-lang/git2-rs.git",
+///     Path::new("/tmp/git2-rs"),
+///   );
+/// ```
 pub struct RepoBuilder<'cb> {
     bare: bool,
     branch: Option<CString>,
@@ -26,8 +61,14 @@ pub struct RepoBuilder<'cb> {
 /// Type of callback passed to `RepoBuilder::remote_create`.
 ///
 /// The second and third arguments are the remote's name and the remote's url.
-pub type RemoteCreate<'cb> = for<'a> FnMut(&'a Repository, &str, &str)
-    -> Result<Remote<'a>, Error> + 'cb;
+pub type RemoteCreate<'cb> =
+    dyn for<'a> FnMut(&'a Repository, &str, &str) -> Result<Remote<'a>, Error> + 'cb;
+
+/// A builder struct for git tree updates, for use with `git_tree_create_updated`.
+pub struct TreeUpdateBuilder {
+    updates: Vec<raw::git_tree_update>,
+    paths: Vec<CString>,
+}
 
 /// A builder struct for configuring checkouts of a repository.
 pub struct CheckoutBuilder<'cb> {
@@ -50,7 +91,7 @@ pub struct CheckoutBuilder<'cb> {
 ///
 /// The first argument is the path for the notification, the next is the numver
 /// of completed steps so far, and the final is the total number of steps.
-pub type Progress<'a> = FnMut(Option<&Path>, usize, usize) + 'a;
+pub type Progress<'a> = dyn FnMut(Option<&Path>, usize, usize) + 'a;
 
 /// Checkout notifications callback.
 ///
@@ -59,10 +100,14 @@ pub type Progress<'a> = FnMut(Option<&Path>, usize, usize) + 'a;
 ///
 /// The callback must return a bool specifying whether the checkout should
 /// continue.
-pub type Notify<'a> = FnMut(CheckoutNotificationType, Option<&Path>,
-                            Option<DiffFile>, Option<DiffFile>,
-                            Option<DiffFile>) -> bool + 'a;
-
+pub type Notify<'a> = dyn FnMut(
+        CheckoutNotificationType,
+        Option<&Path>,
+        Option<DiffFile<'_>>,
+        Option<DiffFile<'_>>,
+        Option<DiffFile<'_>>,
+    ) -> bool
+    + 'a;
 
 impl<'cb> Default for RepoBuilder<'cb> {
     fn default() -> Self {
@@ -98,7 +143,7 @@ impl<'cb> RepoBuilder<'cb> {
     /// When ready, the `clone()` method can be used to clone a new repository
     /// using this configuration.
     pub fn new() -> RepoBuilder<'cb> {
-        ::init();
+        crate::init();
         RepoBuilder {
             bare: false,
             branch: None,
@@ -159,8 +204,7 @@ impl<'cb> RepoBuilder<'cb> {
 
     /// Configure the checkout which will be performed by consuming a checkout
     /// builder.
-    pub fn with_checkout(&mut self, checkout: CheckoutBuilder<'cb>)
-                         -> &mut RepoBuilder<'cb> {
+    pub fn with_checkout(&mut self, checkout: CheckoutBuilder<'cb>) -> &mut RepoBuilder<'cb> {
         self.checkout = Some(checkout);
         self
     }
@@ -169,8 +213,7 @@ impl<'cb> RepoBuilder<'cb> {
     ///
     /// The callbacks are used for reporting fetch progress, and for acquiring
     /// credentials in the event they are needed.
-    pub fn fetch_options(&mut self, fetch_opts: FetchOptions<'cb>)
-                            -> &mut RepoBuilder<'cb> {
+    pub fn fetch_options(&mut self, fetch_opts: FetchOptions<'cb>) -> &mut RepoBuilder<'cb> {
         self.fetch_opts = Some(fetch_opts);
         self
     }
@@ -178,8 +221,8 @@ impl<'cb> RepoBuilder<'cb> {
     /// Configures a callback used to create the git remote, prior to its being
     /// used to perform the clone operation.
     pub fn remote_create<F>(&mut self, f: F) -> &mut RepoBuilder<'cb>
-        where F: for<'a> FnMut(&'a Repository, &str, &str)
-                    -> Result<Remote<'a>, Error> + 'cb,
+    where
+        F: for<'a> FnMut(&'a Repository, &str, &str) -> Result<Remote<'a>, Error> + 'cb,
     {
         self.remote_create = Some(Box::new(f));
         self
@@ -192,13 +235,17 @@ impl<'cb> RepoBuilder<'cb> {
     pub fn clone(&mut self, url: &str, into: &Path) -> Result<Repository, Error> {
         let mut opts: raw::git_clone_options = unsafe { mem::zeroed() };
         unsafe {
-            try_call!(raw::git_clone_init_options(&mut opts,
-                                                  raw::GIT_CLONE_OPTIONS_VERSION));
+            try_call!(raw::git_clone_init_options(
+                &mut opts,
+                raw::GIT_CLONE_OPTIONS_VERSION
+            ));
         }
         opts.bare = self.bare as c_int;
-        opts.checkout_branch = self.branch.as_ref().map(|s| {
-            s.as_ptr()
-        }).unwrap_or(ptr::null());
+        opts.checkout_branch = self
+            .branch
+            .as_ref()
+            .map(|s| s.as_ptr())
+            .unwrap_or(ptr::null());
 
         if let Some(ref local) = self.clone_local {
             opts.local = *local as raw::git_clone_local_t;
@@ -225,8 +272,9 @@ impl<'cb> RepoBuilder<'cb> {
             opts.remote_cb_payload = callback as *mut _ as *mut _;
         }
 
-        let url = try!(CString::new(url));
-        let into = try!(into.into_c_string());
+        let url = CString::new(url)?;
+        // Normal file path OK (does not need Windows conversion).
+        let into = into.into_c_string()?;
         let mut raw = ptr::null_mut();
         unsafe {
             try_call!(raw::git_clone(&mut raw, url, into, &opts));
@@ -235,20 +283,22 @@ impl<'cb> RepoBuilder<'cb> {
     }
 }
 
-extern fn remote_create_cb(out: *mut *mut raw::git_remote,
-                           repo: *mut raw::git_repository,
-                           name: *const c_char,
-                           url: *const c_char,
-                           payload: *mut c_void) -> c_int {
+extern "C" fn remote_create_cb(
+    out: *mut *mut raw::git_remote,
+    repo: *mut raw::git_repository,
+    name: *const c_char,
+    url: *const c_char,
+    payload: *mut c_void,
+) -> c_int {
     unsafe {
         let repo = Repository::from_raw(repo);
         let code = panic::wrap(|| {
             let name = CStr::from_ptr(name).to_str().unwrap();
             let url = CStr::from_ptr(url).to_str().unwrap();
-            let f = payload as *mut Box<RemoteCreate>;
+            let f = payload as *mut Box<RemoteCreate<'_>>;
             match (*f)(&repo, name, url) {
                 Ok(remote) => {
-                    *out = ::remote::remote_into_raw(remote);
+                    *out = crate::remote::remote_into_raw(remote);
                     0
                 }
                 Err(e) => e.raw_code(),
@@ -269,7 +319,7 @@ impl<'cb> CheckoutBuilder<'cb> {
     /// Creates a new builder for checkouts with all of its default
     /// configuration.
     pub fn new() -> CheckoutBuilder<'cb> {
-        ::init();
+        crate::init();
         CheckoutBuilder {
             disable_filters: false,
             dir_perm: None,
@@ -313,8 +363,7 @@ impl<'cb> CheckoutBuilder<'cb> {
         self
     }
 
-    fn flag(&mut self, bit: raw::git_checkout_strategy_t,
-            on: bool) -> &mut CheckoutBuilder<'cb> {
+    fn flag(&mut self, bit: raw::git_checkout_strategy_t, on: bool) -> &mut CheckoutBuilder<'cb> {
         if on {
             self.checkout_opts |= bit as u32;
         } else {
@@ -341,8 +390,7 @@ impl<'cb> CheckoutBuilder<'cb> {
     /// Remove untracked files from the working dir.
     ///
     /// Defaults to false.
-    pub fn remove_untracked(&mut self, remove: bool)
-                            -> &mut CheckoutBuilder<'cb> {
+    pub fn remove_untracked(&mut self, remove: bool) -> &mut CheckoutBuilder<'cb> {
         self.flag(raw::GIT_CHECKOUT_REMOVE_UNTRACKED, remove)
     }
 
@@ -404,16 +452,14 @@ impl<'cb> CheckoutBuilder<'cb> {
     /// Indicate whether ignored files should be overwritten during the checkout.
     ///
     /// Defaults to true.
-    pub fn overwrite_ignored(&mut self, overwrite: bool)
-                             -> &mut CheckoutBuilder<'cb> {
+    pub fn overwrite_ignored(&mut self, overwrite: bool) -> &mut CheckoutBuilder<'cb> {
         self.flag(raw::GIT_CHECKOUT_DONT_OVERWRITE_IGNORED, !overwrite)
     }
 
     /// Indicate whether a normal merge file should be written for conflicts.
     ///
     /// Defaults to false.
-    pub fn conflict_style_merge(&mut self, on: bool)
-                                -> &mut CheckoutBuilder<'cb> {
+    pub fn conflict_style_merge(&mut self, on: bool) -> &mut CheckoutBuilder<'cb> {
         self.flag(raw::GIT_CHECKOUT_CONFLICT_STYLE_MERGE, on)
     }
 
@@ -421,8 +467,10 @@ impl<'cb> CheckoutBuilder<'cb> {
     /// callback.
     ///
     /// Defaults to none.
-    pub fn notify_on(&mut self, notification_types: CheckoutNotificationType)
-                     -> &mut CheckoutBuilder<'cb> {
+    pub fn notify_on(
+        &mut self,
+        notification_types: CheckoutNotificationType,
+    ) -> &mut CheckoutBuilder<'cb> {
         self.notify_flags = notification_types;
         self
     }
@@ -431,14 +479,12 @@ impl<'cb> CheckoutBuilder<'cb> {
     /// for conflicts.
     ///
     /// Defaults to false.
-    pub fn conflict_style_diff3(&mut self, on: bool)
-                                -> &mut CheckoutBuilder<'cb> {
+    pub fn conflict_style_diff3(&mut self, on: bool) -> &mut CheckoutBuilder<'cb> {
         self.flag(raw::GIT_CHECKOUT_CONFLICT_STYLE_DIFF3, on)
     }
 
     /// Indicate whether to apply filters like CRLF conversion.
-    pub fn disable_filters(&mut self, disable: bool)
-                           -> &mut CheckoutBuilder<'cb> {
+    pub fn disable_filters(&mut self, disable: bool) -> &mut CheckoutBuilder<'cb> {
         self.disable_filters = disable;
         self
     }
@@ -463,9 +509,8 @@ impl<'cb> CheckoutBuilder<'cb> {
     ///
     /// If no paths are specified, then all files are checked out. Otherwise
     /// only these specified paths are checked out.
-    pub fn path<T: IntoCString>(&mut self, path: T)
-                                   -> &mut CheckoutBuilder<'cb> {
-        let path = path.into_c_string().unwrap();
+    pub fn path<T: IntoCString>(&mut self, path: T) -> &mut CheckoutBuilder<'cb> {
+        let path = util::cstring_to_repo_path(path).unwrap();
         self.path_ptrs.push(path.as_ptr());
         self.paths.push(path);
         self
@@ -473,6 +518,7 @@ impl<'cb> CheckoutBuilder<'cb> {
 
     /// Set the directory to check out to
     pub fn target_dir(&mut self, dst: &Path) -> &mut CheckoutBuilder<'cb> {
+        // Normal file path OK (does not need Windows conversion).
         self.target_dir = Some(dst.into_c_string().unwrap());
         self
     }
@@ -497,7 +543,9 @@ impl<'cb> CheckoutBuilder<'cb> {
 
     /// Set a callback to receive notifications of checkout progress.
     pub fn progress<F>(&mut self, cb: F) -> &mut CheckoutBuilder<'cb>
-                       where F: FnMut(Option<&Path>, usize, usize) + 'cb {
+    where
+        F: FnMut(Option<&Path>, usize, usize) + 'cb,
+    {
         self.progress = Some(Box::new(cb) as Box<Progress<'cb>>);
         self
     }
@@ -507,8 +555,15 @@ impl<'cb> CheckoutBuilder<'cb> {
     /// Callbacks are invoked prior to modifying any files on disk.
     /// Returning `false` from the callback will cancel the checkout.
     pub fn notify<F>(&mut self, cb: F) -> &mut CheckoutBuilder<'cb>
-        where F: FnMut(CheckoutNotificationType, Option<&Path>, Option<DiffFile>,
-                       Option<DiffFile>, Option<DiffFile>) -> bool + 'cb
+    where
+        F: FnMut(
+                CheckoutNotificationType,
+                Option<&Path>,
+                Option<DiffFile<'_>>,
+                Option<DiffFile<'_>>,
+                Option<DiffFile<'_>>,
+            ) -> bool
+            + 'cb,
     {
         self.notify = Some(Box::new(cb) as Box<Notify<'cb>>);
         self
@@ -542,13 +597,11 @@ impl<'cb> CheckoutBuilder<'cb> {
             opts.their_label = c.as_ptr();
         }
         if self.progress.is_some() {
-            let f: raw::git_checkout_progress_cb = progress_cb;
-            opts.progress_cb = Some(f);
+            opts.progress_cb = Some(progress_cb);
             opts.progress_payload = self as *mut _ as *mut _;
         }
         if self.notify.is_some() {
-            let f: raw::git_checkout_notify_cb = notify_cb;
-            opts.notify_cb = Some(f);
+            opts.notify_cb = Some(notify_cb);
             opts.notify_payload = self as *mut _ as *mut _;
             opts.notify_flags = self.notify_flags.bits() as c_uint;
         }
@@ -556,12 +609,14 @@ impl<'cb> CheckoutBuilder<'cb> {
     }
 }
 
-extern fn progress_cb(path: *const c_char,
-                      completed: size_t,
-                      total: size_t,
-                      data: *mut c_void) {
+extern "C" fn progress_cb(
+    path: *const c_char,
+    completed: size_t,
+    total: size_t,
+    data: *mut c_void,
+) {
     panic::wrap(|| unsafe {
-        let payload = &mut *(data as *mut CheckoutBuilder);
+        let payload = &mut *(data as *mut CheckoutBuilder<'_>);
         let callback = match payload.progress {
             Some(ref mut c) => c,
             None => return,
@@ -575,15 +630,17 @@ extern fn progress_cb(path: *const c_char,
     });
 }
 
-extern fn notify_cb(why: raw::git_checkout_notify_t,
-                    path: *const c_char,
-                    baseline: *const raw::git_diff_file,
-                    target: *const raw::git_diff_file,
-                    workdir: *const raw::git_diff_file,
-                    data: *mut c_void) -> c_int {
+extern "C" fn notify_cb(
+    why: raw::git_checkout_notify_t,
+    path: *const c_char,
+    baseline: *const raw::git_diff_file,
+    target: *const raw::git_diff_file,
+    workdir: *const raw::git_diff_file,
+    data: *mut c_void,
+) -> c_int {
     // pack callback etc
     panic::wrap(|| unsafe {
-        let payload = &mut *(data as *mut CheckoutBuilder);
+        let payload = &mut *(data as *mut CheckoutBuilder<'_>);
         let callback = match payload.notify {
             Some(ref mut c) => c,
             None => return 0,
@@ -614,17 +671,93 @@ extern fn notify_cb(why: raw::git_checkout_notify_t,
 
         let why = CheckoutNotificationType::from_bits_truncate(why as u32);
         let keep_going = callback(why, path, baseline, target, workdir);
-        if keep_going {0} else {1}
-    }).unwrap_or(2)
+        if keep_going {
+            0
+        } else {
+            1
+        }
+    })
+    .unwrap_or(2)
+}
+
+unsafe impl Send for TreeUpdateBuilder {}
+
+impl Default for TreeUpdateBuilder {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
+impl TreeUpdateBuilder {
+    /// Create a new empty series of updates.
+    pub fn new() -> Self {
+        Self {
+            updates: Vec::new(),
+            paths: Vec::new(),
+        }
+    }
+
+    /// Add an update removing the specified `path` from a tree.
+    pub fn remove<T: IntoCString>(&mut self, path: T) -> &mut Self {
+        let path = util::cstring_to_repo_path(path).unwrap();
+        let path_ptr = path.as_ptr();
+        self.paths.push(path);
+        self.updates.push(raw::git_tree_update {
+            action: raw::GIT_TREE_UPDATE_REMOVE,
+            id: raw::git_oid {
+                id: [0; raw::GIT_OID_RAWSZ],
+            },
+            filemode: raw::GIT_FILEMODE_UNREADABLE,
+            path: path_ptr,
+        });
+        self
+    }
+
+    /// Add an update setting the specified `path` to a specific Oid, whether it currently exists
+    /// or not.
+    ///
+    /// Note that libgit2 does not support an upsert of a previously removed path, or an upsert
+    /// that changes the type of an object (such as from tree to blob or vice versa).
+    pub fn upsert<T: IntoCString>(&mut self, path: T, id: Oid, filemode: FileMode) -> &mut Self {
+        let path = util::cstring_to_repo_path(path).unwrap();
+        let path_ptr = path.as_ptr();
+        self.paths.push(path);
+        self.updates.push(raw::git_tree_update {
+            action: raw::GIT_TREE_UPDATE_UPSERT,
+            id: unsafe { *id.raw() },
+            filemode: u32::from(filemode) as raw::git_filemode_t,
+            path: path_ptr,
+        });
+        self
+    }
+
+    /// Create a new tree from the specified baseline and this series of updates.
+    ///
+    /// The baseline tree must exist in the specified repository.
+    pub fn create_updated(&mut self, repo: &Repository, baseline: &Tree<'_>) -> Result<Oid, Error> {
+        let mut ret = raw::git_oid {
+            id: [0; raw::GIT_OID_RAWSZ],
+        };
+        unsafe {
+            try_call!(raw::git_tree_create_updated(
+                &mut ret,
+                repo.raw(),
+                baseline.raw(),
+                self.updates.len(),
+                self.updates.as_ptr()
+            ));
+            Ok(Binding::from_raw(&ret as *const _))
+        }
+    }
 }
 
 #[cfg(test)]
 mod tests {
+    use super::{CheckoutBuilder, RepoBuilder, TreeUpdateBuilder};
+    use crate::{CheckoutNotificationType, FileMode, Repository};
     use std::fs;
     use std::path::Path;
-    use tempdir::TempDir;
-    use super::{CheckoutBuilder, RepoBuilder};
-    use {CheckoutNotificationType, Repository};
+    use tempfile::TempDir;
 
     #[test]
     fn smoke() {
@@ -634,30 +767,50 @@ mod tests {
 
     #[test]
     fn smoke2() {
-        let td = TempDir::new("test").unwrap();
+        let td = TempDir::new().unwrap();
         Repository::init_bare(&td.path().join("bare")).unwrap();
         let url = if cfg!(unix) {
             format!("file://{}/bare", td.path().display())
         } else {
-            format!("file:///{}/bare", td.path().display().to_string()
-                                         .replace("\\", "/"))
+            format!(
+                "file:///{}/bare",
+                td.path().display().to_string().replace("\\", "/")
+            )
         };
 
         let dst = td.path().join("foo");
         RepoBuilder::new().clone(&url, &dst).unwrap();
         fs::remove_dir_all(&dst).unwrap();
-        assert!(RepoBuilder::new().branch("foo")
-                                  .clone(&url, &dst).is_err());
+        assert!(RepoBuilder::new().branch("foo").clone(&url, &dst).is_err());
+    }
+
+    #[test]
+    fn smoke_tree_create_updated() {
+        let (_tempdir, repo) = crate::test::repo_init();
+        let (_, tree_id) = crate::test::commit(&repo);
+        let tree = t!(repo.find_tree(tree_id));
+        assert!(tree.get_name("bar").is_none());
+        let foo_id = tree.get_name("foo").unwrap().id();
+        let tree2_id = t!(TreeUpdateBuilder::new()
+            .remove("foo")
+            .upsert("bar/baz", foo_id, FileMode::Blob)
+            .create_updated(&repo, &tree));
+        let tree2 = t!(repo.find_tree(tree2_id));
+        assert!(tree2.get_name("foo").is_none());
+        let baz_id = tree2.get_path(Path::new("bar/baz")).unwrap().id();
+        assert_eq!(foo_id, baz_id);
     }
 
     /// Issue regression test #365
     #[test]
     fn notify_callback() {
-        let td = TempDir::new("test").unwrap();
-        let cd = TempDir::new("external-checkout").unwrap();
+        let td = TempDir::new().unwrap();
+        let cd = TempDir::new().unwrap();
 
         {
-            let repo = Repository::init(&td.path()).unwrap();
+            let mut opts = crate::RepositoryInitOptions::new();
+            opts.initial_head("main");
+            let repo = Repository::init_opts(&td.path(), &opts).unwrap();
 
             let mut config = repo.config().unwrap();
             config.set_str("user.name", "name").unwrap();
@@ -672,13 +825,13 @@ mod tests {
 
             let tree = repo.find_tree(id).unwrap();
             let sig = repo.signature().unwrap();
-            repo.commit(Some("HEAD"), &sig, &sig, "initial",
-            &tree, &[]).unwrap();
+            repo.commit(Some("HEAD"), &sig, &sig, "initial", &tree, &[])
+                .unwrap();
         }
 
         let repo = Repository::open_bare(&td.path().join(".git")).unwrap();
         let tree = repo
-            .revparse_single(&"master")
+            .revparse_single(&"main")
             .unwrap()
             .peel_to_tree()
             .unwrap();
@@ -697,5 +850,4 @@ mod tests {
         repo.checkout_index(Some(&mut index), Some(&mut checkout_opts))
             .unwrap();
     }
-
 }
